function [beta,alpha,gamma,lambda,f,exitflag,obj,APE,Var_beta,Var_APE,beta_corrected,APE_corrected, v_reference] = FactorMLE(dist,Y,X,weight,lambda_known,f_known,R,precision_beta,repMIN,repMAX,MAX_STEPS,beta_start,alpha_start,gamma_start,lambda_start,f_start)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%%% Maximum Likelihood Estimation of Panel Factor Models
 %%%% This version October 6th
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % INPUT PARAMETERS: 
 %     dist = 'probit' for probit model
 %          = 'logit' for logit model
 %          = 'LS' for (weighted) least squares estimation (i.e. Gaussian MLE)
 %          = 'Poisson' for Poisson
 %     Y = NxT matrix of outcomes
 %     X = KxNxT multi-matrix of regressors
 %     weight = NxT matrix that specifies fixed weights for each observation,
 %              for standard MLE we set weight(i,t)=1 if (i,t) observed
 %              and weight(i,t)=0 if (i,t) unobserved,
 %              but more general weights are also possible.
 %     lambda_known = N x Rex1 matrix of KNOWN factor loadings,
 %                    e.g. lambda_known=ones(N,1) to control standard time dummies
 %         ... if NO KNOWN factor loadings, then set lambda_known=zeros(N,0) !!!
 %     f_known = T x Rex2 matrix of KNOWN factors,
 %                    e.g. f_known=ones(T,1) to control standard individual specific fixed effects
 %         ... if NO KNOWN factors, then set f_known=zeros(T,0) !!!
 %     R = positive integer 
 %         ... number of interactive fixed effects in the estimation
 %     precision_beta = defines stopping criteria for numerical optimization,
 %                      namely optimization is stopped when difference in beta
 %                      relative to previous opimtization step is smaller than
 %                      "precision_beta" (uniformly over all K components of beta)
 %     repMIN = positive integer
 %              ... minimal number of runs of optimization with different starting points
 %     repMAX = positive integer
 %              ... maximal number of runs of optimization (in case numerical optimization
 %              doesn't terminate properly, we do multiple runs even for repMIN=1)
 %     MAX_STEPS = maximum number of iteration steps for each optimization
 %                 run (e.g. MAX_STEPS=300)
 %
 % OPTIONAL INPUT PARAMTERS:
 %     beta_start, alpha_start, gamma_start, lambda_start, f_start
 %           ... starting values for first numerical optimization
 %               (afterwards random starting values)
 %           ... same dimensions as output parameters below
 %
 % OUTPUT PARAMETERS:
 %     beta   = Kx1 vector of parameter estimate 
 %     alpha  = N x Rex2 matrix of factor loadings corresponding to f_known
 %     gamma  = T x Rex1 matrix of factors corresponding to lambda_known 
 %     lambda = N x R matrix of estimates for factor loading
 %     f      = T x R matrix of estimates for factors
 %     exitflag = 1 if iteration algorithm properly converged at optimal beta
 %              = -1 if iteration algorithm did not properly converge at optimal beta
 %     obj = objective function at optimum
 %     Var_beta = estimated variance-covariance matrix of beta
 %     APE = Kx1 vector of average partial effects
 %     Var_APE = estimated variance-covariance matrix of APE's
 %     beta_corrected = bias corrected estimator for beta
 %     APE_corrected = bias corrected estimator for APE's
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
  K    = size(X,1);   %number of regressors
  N    = size(X,2);   %cross-sectional dimension
  T    = size(X,3);   %time-serial dimension    
  Rex1 = size(lambda_known,2);
  Rex2 = size(f_known,2);
     
  %check that either 'logit' or 'probit' is specified
  if ~( strcmp(dist,'logit') | strcmp(dist,'probit') | strcmp(dist,'LS') | strcmp(dist, 'Poisson'))
    disp('ERROR in FactorMLE: first input argument needs to be ''logit'' or ''probit'' or "Poisson" or ''LS''.');
    exitflag=-1;
    return
  end    
  
  %if user provided starting values, then use those:
  if nargin>11        
    beta=beta_start;
    alpha=alpha_start;
    gamma=gamma_start;
    lambda=lambda_start;
    f=f_start;
    if (size(beta,1)~=K) | (size(beta,2)~=1) | ...
       (size(alpha,1)~=N) | (size(alpha,2)~=size(f_known,2)) | ... 
       (size(gamma,1)~=T) | (size(gamma,2)~=size(lambda_known,2)) | ...
       (size(lambda,1)~=N) | (size(lambda,2)~=R) | ...
       (size(f,1)~=T) | (size(f,2)~=R)
   
       size(beta)
       size(alpha)
       size(gamma)
       size(lambda)
       size(f)
   
       exitflag=-1;
       disp('ERROR in FactorMLE: PARAMETER STARTING VALUES HAVE WRONG DIMENSIONS');
       return;
    end 
  else                %otherwise the first iteration starts at zero:
    beta=zeros(K,1);
    alpha=zeros(N,size(f_known,2));
    gamma=zeros(T,size(lambda_known,2));
    lambda=zeros(N,R);
    f=zeros(T,R);
  end  
  
  %make sure that all regressor values for missing data are equal to zero:
  for i=1:N
      for t=1:T    
        if weight(i,t)==0
          X(:,i,t)=0;
        end
      end
  end
  
  if nargout>8
    variance_requested=1;   %user wants variance
  else
    variance_requested=0;
  end    
  
  %NUMERICAL OPTIMIZATION:
  
  obj_best=-inf;
  
  count_successful_runs = 0;
  count_total_runs      = 0;
  while (count_successful_runs<repMIN) && (count_total_runs<repMAX)
    count_total_runs=count_total_runs+1; 
      
    %ITERATION:
    step_size   = inf;
    count       = 0;
    count_final = 0;  %counter how many steps already made smaller than precision_beta
    v_reference = [];
    para_reference=[];
    b_reference=0;
    while (count_final<=5) && (count<MAX_STEPS)   % repeart iteration step as long beta still changes more than "precision_beta", or after MAX_STEPS steps
      count=count+1
      betaOLD=beta;
      
      if count<=5
        [beta,alpha,gamma,lambda,f,obj]=step(dist,Y,X,weight,lambda_known,f_known,beta,alpha,gamma,lambda,f); 
      else  
        [beta,alpha,gamma,lambda,f,obj,v_reference,para_reference,b_reference]=stepNR(dist,Y,X,weight,lambda_known,f_known,beta,alpha,gamma,lambda,f,v_reference,para_reference,b_reference); 
      end  
      
      step_size=max(abs(beta-betaOLD));
      if step_size <= precision_beta
        count_final=count_final+1;
      else
        count_final=0; 
      end    
      
      format long
      obj
    end  
    
    %COUNT SUCCESSFUL ITERATIONS:
    status=-1;
    if count<MAX_STEPS        %an iteration is succesful if stopped before MAX_STEPS is reached
      count_successful_runs = count_successful_runs+1;
      status=1;
    end
    
    %CHECK IF OBJECTIVE IS BETTER THAN PREVIOUSLY BEST OBJECTIVE,
    %IF SO, THEN SAVE THOSE PARAMETERS: 
    if obj>obj_best
      obj_best    = obj; 
      beta_best   = beta;
      alpha_best  = alpha;
      gamma_best  = gamma;
      lambda_best = lambda;
      f_best      = f;
      exitflag    = status;   %exitflag reports if those optimal parameter values correspond to "successful" iteration run, with above definition of "successful"
    end    

    %GENERATE NEW RANDOM STARTING VALUES:
    beta=2*rand(K,1)-1;
    alpha=2*rand(N,size(f_known,2))-1;
    gamma=2*rand(T,size(lambda_known,2))-1;
    lambda=2*rand(N,R)-1;
    f=2*rand(T,R)-1;
  end  

  %REPORT PARAMETERS WITH MAXIMUM OBJECTIVE THAT WAS FOUND:
  beta=beta_best;
  alpha=alpha_best;
  gamma=gamma_best;
  lambda=lambda_best;
  f=f_best;
    
  %CALCULATE VARAINCE OF ESTIMATOR VIA HESSIAN:
  
  if variance_requested==1
    [obj,score,Hessian]=SampleLogL(dist,Y,X,weight,lambda_known,f_known,beta,alpha,gamma,lambda,f,0);
    V=pinv(-Hessian * sum(sum(weight)) );
    Var_beta=V(1:K,1:K);
             %  obj
             %  mx_score=max(abs(score))
  else
    obj=SampleLogL(dist,Y,X,weight,lambda_known,f_known,beta,alpha,gamma,lambda,f,0); 
  end

  %CALCULATE AVERAGE PARTIAL EFFECTS:

  Z = mult(X,beta) + alpha * f_known' + lambda_known * gamma' + lambda * f';
  if strcmp(dist,'logit') | strcmp(dist,'probit')| strcmp(dist,'Poisson')
    %%%%%
    %calculatrion of APE for CONTINUOUS REGRESSORS (IN FACT WE USE THIS FOR ALL NON-BINARY REGRESSORS):
    %%%%%
    if strcmp(dist,'logit') 
      Z          = max(min(Z,100),-100);               %truncate index at -100 and 100
      bc_pdf     = exp(Z) ./  (1+exp(Z)).^2;
      bc_der_pdf = exp(Z) .* (1-exp(Z)) ./  (1+exp(Z)).^3;
    elseif strcmp(dist, 'probit')
      Z          = max(min(Z,6),-6);               %truncate index at -6 and 6
      bc_pdf     = normal_pdf(Z);
      bc_der_pdf = normal_pdf_derivative(Z);
    else 
      bc_pdf     = exp(Z);
      bc_der_pdf = exp(Z);
    end  
    
    bc_pdf     = weight .*bc_pdf;
    bc_der_pdf = weight .*bc_der_pdf;
    
    F1         = sum(sum(bc_pdf)) / sum(sum(weight));
    APE_CONT   = beta * F1;
    
    %calculate variance of APE by delta method (using Jacobian J)
    if variance_requested==1    
        s  = zeros(K + N*(R+Rex2) + T*(R+Rex1),1);  %score of F1 = average(F'(Z))
        ff = [f,f_known];
        ll = [lambda,lambda_known];
        for k=1:K
            XX   = squeeze(X(k,:,:));
            s(k) = sum(sum(bc_der_pdf .* XX))/sum(sum(weight));
        end
        for r=1:R+Rex2
            s(K + (r-1)*N + (1:N))              =  bc_der_pdf * ff(:,r)/sum(sum(weight));
        end
        for r=1:R+Rex1
            s(K + N*(R+Rex2) + (r-1)*T + (1:T)) =  bc_der_pdf' * ll(:,r)/sum(sum(weight));
        end
        
        J            = eye(K+N*(R+Rex2)+T*(R+Rex1), K)*F1 + s*ones(1,K);
        Var_APE_CONT = J'*V*J;
    end
    
     %%%%%
     %calculatrion of APE for BINARY REGRESSORS:
     %%%%%
     APE = APE_CONT; 
     if variance_requested==1 
         Var_APE=diag(Var_APE_CONT);   
     end 
    
%      for k=1:K                        %go through all regressors
%          if X(k,:,:) == (X(k,:,:)==1)   %if regressors is BINARY;
%             X0        = X;
%             X0(k,:,:) = 0; 
%             Z0 = mult(X0,beta) + alpha * f_known' + lambda_known * gamma' + lambda * f';    %index with k'th regressors=0
%         
%             X1        = X;
%             X1(k,:,:) = 1;   
%             Z1 = mult(X1,beta) + alpha * f_known' + lambda_known * gamma' + lambda * f';     %index with k'th regressors=1
%             
%             if strcmp(dist,'logit') 
%                 Z0      = max(min(Z0,100),-100);               %truncate index at -100 and 100
%                 Z1      = max(min(Z1,100),-100);               %truncate index at -100 and 100
%                 bc_cdf0 = exp(Z0) ./  (1+exp(Z0));
%                 bc_pdf0 = exp(Z0) ./  (1+exp(Z0)).^2;
%                
%                 bc_cdf1 = exp(Z1) ./  (1+exp(Z1));
%                 bc_pdf1 = exp(Z1) ./  (1+exp(Z1)).^2;
%            
%             elseif strcmp(dist, 'probit')
%                 Z0      = max(min(Z0,6),-6);               %truncate index at -6 and 6
%                 Z1      = max(min(Z1,6),-6);               %truncate index at -6 and 6
%                 bc_cdf0 = normal_cdf(Z0);
%                 bc_pdf0 = normal_pdf(Z0);
%                
%                 bc_cdf1 = normal_cdf(Z1);
%                 bc_pdf1 = normal_pdf(Z1);
%             else
%                  
%                 bc_cdf0 = exp(Z0);
%                 bc_pdf0 = exp(Z0);
%                
%                 bc_cdf1 = exp(Z1);
%                 bc_pdf1 = exp(Z1);
%             end
%             
%             bc_cdf0 = weight.*bc_cdf0;
%             bc_pdf0 = weight.*bc_pdf0;
%             bc_cdf1 = weight.*bc_cdf1;
%             bc_pdf1 = weight.*bc_pdf1;
%          
%             APE(k) = sum(sum( bc_cdf1-bc_cdf0  )) / sum(sum(weight));   %partial effect is average outcome at X_k=1 minus average outcome at X_k=0
%     
            %calculate variance of APE by delta method:
%             if variance_requested==1    
%                 grad=zeros(K + N*(R+Rex2) + T*(R+Rex1),1);  %gradient of APE(k) wrt all parameters
%                 ff=[f,f_known];
%                 ll=[lambda,lambda_known];
%                 for k=1:K
%                     XX0=squeeze(X0(k,:,:));
%                     XX1=squeeze(X1(k,:,:));
%                     grad(k)=sum(sum(bc_pdf1 .* XX1 - bc_pdf0 .* XX0)) / sum(sum(weight));
%                 end
%                 for r=1:R+Rex2
%                     s(K + (r-1)*N + (1:N)) =  (bc_pdf1 - bc_pdf0)  * ff(:,r) / sum(sum(weight));
%                 end
%                 for r=1:R+Rex1
%                     s(K + N*(R+Rex2) + (r-1)*T + (1:T)) =  (bc_pdf1 - bc_pdf0)' * ll(:,r) / sum(sum(weight));
%                 end
% 
%                 Var_APE(k)=grad'*V*grad;
%             end       
%         end
%      end
    
  else
    APE = beta; 
    if variance_requested==1
      Var_APE = diag(Var_beta);
    end  
  end    
  
  
  %CALCULATE BIAS CORRECTED ESTIMATORS VIA JACKKNIFE (assuming N=T for trade application)
  if nargout>10
      if N==T
          repMIN2         = 1;
          repMAX2         = 1;
          MAX_STEPS2      = 150;
          precision_beta2 = precision_beta;
          for i=1:N
              disp(['Jackknife run ' num2str(i) ' / ' num2str(N)])
              
              Y2=Y; X2=X; weight2=weight; 
              Y2(i,:)=[];  Y2(:,i)=[];
              X2(:,i,:)=[];  X2(:,:,i)=[];
              weight2(i,:)=[];  weight2(:,i)=[];
              
              beta2=beta; alpha2=alpha; gamma2=gamma; lambda2=lambda; f2=f;
              lambda_known2=lambda_known; f_known2=f_known;
              alpha2(i,:)=[];  gamma2(i,:)=[];  lambda2(i,:)=[];  f2(i,:)=[];
              lambda_known2(i,:)=[];  f_known2(i,:)=[];
              
              [beta2,alpha2,gamma2,lambda2,f2,exitflag2,obj2,APE2] = FactorMLE(dist,Y2,X2,weight2,lambda_known2,f_known2,R,precision_beta2,repMIN2,repMAX2,MAX_STEPS2,beta2,alpha2,gamma2,lambda2,f2);
              betaJK(:,i) = beta2;
              APEJK(:,i)  = APE2;
          end
          beta_corrected = N*beta - (N-1)*mean(betaJK,2);
          APE_corrected = N*APE - (N-1)*mean(APEJK,2);
      else
        beta_corrected = zeros(K,1);
        APE_corrected  = zeros(K,1);
      end
  end
      
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% define observation specific log-likelihood, score, Hessian:
%%%% (y is the outcome, z is the single index)
%%%% (inputs y and z can also be vectors or matrices, in which case 
%%%%  the outputs logL, dlogL, ddlogL have the same dimension)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [logL,dlogL,ddlogL]=model(dist,y,z,weight)
  if strcmp(dist,'logit')
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %LOGIT LOG LIKELIHOOD AND DERIVATIVES:
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    z=max(min(z,100),-100);               %truncate index at -100 and 100
    logL   = y.*z - log(1+exp(z));        %log-likelihood for each observation
    %if user also requestes derivatives:
    if nargout>1
      dlogL  = y - exp(z) ./ (1+exp(z));    %score wrt single index for each observation
      ddlogL = - exp(z) ./ ((1+exp(z)).^2); %Hessian wrt single index for each observation
    end
     
  elseif strcmp(dist,'probit')
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %PROBIT LOG LIKELIHOOD AND DERIVATIVES:    
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    z=max(min(z,6),-6);               %truncate index at -6 and 6
    F=normal_cdf(z);
    logL=y.*log(F)+(1-y).*log(1-F);           %log-likelihood for each observation
    if nargout>1
      F1=normal_pdf(z);
      dlogL=y.*F1./F-(1-y).*F1./(1-F);
      F2=normal_pdf_derivative(z);
      ddlogL=y.*(F2./F-F1.^2./F.^2)-(1-y).*(F2./(1-F)+F1.^2./(1-F).^2);
    end
    
  elseif strcmp(dist,'Poisson')
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %POISSON LOG LIKELIHOOD AND DERIVATIVES:    
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    logL = y.*z - exp(z);
    if nargout > 1
        dlogL  = (y - exp(z));
        ddlogL = - exp(z);
    end
     
  else
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %MINUS LEAST SQUARES OBJECTIVE AND DERIVATIVES (i.e. STANDARD NORMAL LOG LIKELIHOOD) 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    logL = - (y-z).^2 / 2;  
    if nargout>1
      dlogL  = (y-z);
      ddlogL = - 1 + 0*y;   % 0*y only to guarantee ddlogL has same dimensions as y
    end    
  end

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %FINALLY, WE WEIGH ALL OBSERVATIONS: (in particular, this gives zero
  %weight to missing observation with weight(i,t)=0)
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  logL = weight.*logL;
  if nargout>1
    dlogL  = weight.*dlogL;  
    ddlogL = weight.*ddlogL;      
  end    
return 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% calculate full sample log-likelihood, score, Hessian:
%%%% NOTE: we do not include "dlogL" terms in the Hessian if Hlin=0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [obj,s,H]=SampleLogL(dist,Y,X,weight,lambda_known,f_known,beta,alpha,gamma,lambda,f,Hlin,logL,dlogL,ddlogL)
  %score and Hessian use the following order of parameters:
  %beta, lambda, alpha, f, gamma
    
  K    = size(X,1);
  N    = size(X,2);
  T    = size(X,3);
  R    = size(lambda,2);
  Rex1 = size(lambda_known,2);
  Rex2 = size(f_known,2);

  ff   = [f,f_known];
  ll   = [lambda,lambda_known];
  
  if nargin<=12  %if logL, dlogL, ddlogL not already provided as inputs, then compute them
    Z                   = mult(X,beta) + alpha * f_known' + lambda_known * gamma' + lambda * f';
    [logL,dlogL,ddlogL] = model(dist,Y,Z,weight);
  end  

  %calculate log-likelihood:
  obj=sum(sum(logL))/N/T;
  
  if nargout==1
    return  
  end    
 
  %calculate score:
  s = zeros(K + N*(R+Rex2) + T*(R+Rex1),1); 
  for k=1:K
    XX   = squeeze(X(k,:,:));  
    s(k) = sum(sum(dlogL .* XX))/N/T;
  end    
  for r=1:R+Rex2
    s(K + (r-1)*N + (1:N)) =  dlogL * ff(:,r)/N/T;
  end
  for r=1:R+Rex1
    s(K + N*(R+Rex2) + (r-1)*T + (1:T)) =  dlogL' * ll(:,r)/N/T;
  end
  
  if nargout==2
    return  
  end    
  
  %calculate Hessian:
  H = zeros(K + N*(R+Rex2) + T*(R+Rex1), K + N*(R+Rex2) + T*(R+Rex1)); 
  for k1=1:K
    X1          = squeeze(X(k1,:,:));  
    for k2=1:K
      X2        = squeeze(X(k2,:,:));  
      H(k1,k2)  = sum(sum(ddlogL .* X1 .* X2))/N/T;
    end  
    for r = 1:R+Rex2
      ind       =  K + (r-1)*N + (1:N);
      H(k1,ind) =  (X1.*ddlogL) * ff(:,r)/N/T;
      H(ind,k1) =  H(k1,ind)';
    end
    for r = 1:R+Rex1
      ind       =  K + N*(R+Rex2) + (r-1)*T + (1:T);  
      H(k1,ind) =  (X1.*ddlogL)' * ll(:,r)/N/T;
      H(ind,k1) =  H(k1,ind)';
    end
  end
  
  for r1 = 1:R+Rex2
    ind1           = K + (r1-1)*N + (1:N);  
    for r2 = 1:R+Rex2
      ind2         = K + (r2-1)*N + (1:N);  
      H(ind1,ind2) = diag(ddlogL * (ff(:,r1).*ff(:,r2)) /N/T );
    end
  end
  
  for r1 = 1:R+Rex1
    ind1           = K + N*(R+Rex2) + (r1-1)*T + (1:T);
    for r2 = 1:R+Rex1
      ind2         = K + N*(R+Rex2) + (r2-1)*T + (1:T);
      H(ind1,ind2) = diag(ddlogL' * (ll(:,r1).*ll(:,r2)) /N/T);
    end
  end

  for r1 = 1:R+Rex2
    ind1           = K + (r1-1)*N + (1:N);
    for r2 = 1:R+Rex1
      ind2         = K + N*(R+Rex2) + (r2-1)*T + (1:T); 
      H(ind1,ind2) = ddlogL .* ( ll(:,r2) * ff(:,r1)' )/N/T;
      H(ind2,ind1) = H(ind1,ind2)';
    end
  end
  
  if Hlin==1     %add terms in the Hessian, which are only there because index itself is non-linear.
                 %those terms depend on the first deriavtive of the
                 %log-likelihood (dlogL)         
    for r=1:R
      ind1         = K + (r-1)*N + (1:N);
      ind2         = K + N*(R+Rex2) + (r-1)*T + (1:T); 
      H(ind1,ind2) = H(ind1,ind2) + dlogL/N/T;
      H(ind2,ind1) = H(ind2,ind1) + dlogL'/N/T;
    end  
  end  
  
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Some helper functions that make it easier to define
%%%% the penalized objective function below:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%translate parameter vector of length K + N*(R+Rex2) + T*(R+Rex1) back into
%the original parameters:
function [beta,alpha,gamma,lambda,f] = para_transform1(para,K,N,T,R,Rex1,Rex2)
    beta = para(1:K);
    lambda=zeros(N,R);
    f=zeros(T,R);
    alpha=zeros(N,Rex2);
    gamma=zeros(T,Rex1);
  for r = 1:R
    lambda(:,r) = para(K + (r-1)*N + (1:N)) * sqrt(N);
  end
  for r = 1:Rex2
    alpha(:,r)  = para(K + (R+r-1)*N + (1:N)) * sqrt(N);
  end
  for r = 1:R
    f(:,r)      = para(K + N*(R+Rex2) + (r-1)*T + (1:T)) * sqrt(T);
  end
  for r = 1:Rex1
    gamma(:,r)  = para(K + N*(R+Rex2) + (R+r-1)*T + (1:T)) * sqrt(T);
  end
return    

%translate original parameters into single parameter vector of length 
%K + N*(R+Rex2) + T*(R+Rex1)
function para=para_transform2(beta,alpha,gamma,lambda,f)
   K    = length(beta);
   N    = size(lambda,1);
   T    = size(f,1);   
   R    = size(lambda,2);
   Rex1 = size(gamma,2);
   Rex2 = size(alpha,2);
   
   para = zeros(K + N*(R+Rex2) + T*(R+Rex1),  1);
   
   para(1:K)=beta;
   for r=1:R
     para(K + (r-1)*N + (1:N))                = lambda(:,r)/sqrt(N);
   end
   for r=1:Rex2
     para(K + (R+r-1)*N + (1:N))              = alpha(:,r)/sqrt(N);
   end
   for r=1:R
     para(K + N*(R+Rex2) + (r-1)*T + (1:T))   = f(:,r)/sqrt(T);
   end
   for r=1:Rex1
     para(K + N*(R+Rex2) + (R+r-1)*T + (1:T)) = gamma(:,r)/sqrt(T);
   end   
return      

%define the matrix of flat directions of the likelihood:
function v = flat_directions(alpha,gamma,lambda,f,lambda_known,f_known,K)
  N   = size(lambda,1);
  T   = size(f,1);
  R   = size(lambda,2);
  Rex1= size(lambda_known,2);
  Rex2= size(f_known,2);

  ff  = [f,f_known];
  ll  = [lambda,lambda_known];
  v   = zeros(K + N*(R+Rex2) + T*(R+Rex1),  (R+Rex2) * (R+Rex1));
  cnt = 0;
  for r1=1:R+Rex2
    for r2=1:R+Rex1
        ind1    = K + (r1-1)*N + (1:N);  
        ind2    = K + N*(R+Rex2) + (r2-1)*T + (1:T);
        vv      = zeros(K + N*(R+Rex2) + T*(R+Rex1),1);
        vv(ind1)= ll(:,r2);
        vv(ind2)= -ff(:,r1);
        vv      = vv/norm(vv);
        cnt     = cnt+1;
        v(:,cnt)= vv;
    end
  end
return

%normalize parameters:
function [alpha,gamma,lambda,f] = normalize(alpha,gamma,lambda,f,lambda_known,f_known)
  
   N              = size(lambda,1);
   T              = size(f,1);

   M_lambda_known = eye(N)-lambda_known/(lambda_known'*lambda_known)*lambda_known';
   M_f_known      = eye(T)-f_known/(f_known'*f_known)*f_known';
   
   %find normalized alpha and gamma:
   Z      = alpha * f_known' + lambda_known * gamma' + lambda * f';
   Z      = Z - M_lambda_known * Z * M_f_known;   %part of index that can be explained by lambda_known and f_known
   gamma  = Z'*lambda_known/(lambda_known'*lambda_known);
   Z      = Z-lambda_known * gamma';
   alpha  = Z*f_known/(f_known'*f_known);

   %find normalized lambda and f:
   f      = M_f_known * f;
   lambda = M_lambda_known * lambda;   %we already took care of everything that can be explained by f_known and lambda_known
   
   A      = sqrtm(f'*f/T);  %note that this is symmetric
   f      = f/A;
   lambda = lambda*A;
 return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% perform one iteration step using Newton-Raphson
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [beta,alpha,gamma,lambda,f,obj,v_reference,para_reference,b_reference] = stepNR(dist,Y,X,weight,lambda_known,f_known,beta0,alpha0,gamma0,lambda0,f0,v_reference,para_reference,b_reference)
  
   K    = size(X,1);
   N    = size(X,2);
   T    = size(X,3);   
   R    = size(lambda0,2);
   Rex1 = size(lambda_known,2);
   Rex2 = size(f_known,2);
   
   %get log-likelihood, score, Hessian for each observation:
   [obj0,score0,Hessian0] = SampleLogL(dist,Y,X,weight,lambda_known,f_known,beta0,alpha0,gamma0,lambda0,f0,1);
   
   %rescale score and Hessian to account for parameter rescaling:
   rescale  = [ones(K,1); sqrt(N)*ones(N*(R+Rex2),1); sqrt(T)*ones(T*(R+Rex1),1)];
   score0   = rescale .* score0;
   Hessian0 = (rescale*rescale') .* Hessian0;  

   %check if v_reference is still appropriate
   if b_reference>0
     v      = flat_directions(alpha0,gamma0,lambda0,f0,lambda_known,f_known,K);
     ev_min = min(abs(eig(v'*v_reference*v_reference'*v)));
   else
     ev_min = 0;
   end  
   
   if ev_min < 0.1   %criterion for updating reference parameter
     disp('updating reference parameter for penalized objective function');  
     [alpha0,gamma0,lambda0,f0] = normalize(alpha0,gamma0,lambda0,f0,lambda_known,f_known);
     para_reference             = para_transform2(beta0,alpha0,gamma0,lambda0,f0);
     v_reference                = flat_directions(alpha0,gamma0,lambda0,f0,lambda_known,f_known,K);
     b_reference                = abs(trace(-Hessian0)/size(Hessian0,1));
   end   
   
   %penalized objective:
   para0   = para_transform2(beta0,alpha0,gamma0,lambda0,f0);
   objP    = obj0 - b_reference/2 * ((para0-para_reference)' * v_reference) * (v_reference' * (para0-para_reference));
   score   = score0 - b_reference * v_reference * (v_reference' * (para0-para_reference));
   Hessian = Hessian0 - b_reference * v_reference * v_reference';
   
   epsilon_max = 1;
   epsilon_min = 10^(-4);  
   epsilon     = 10^(-5);
   
   stop2=0;
   while stop2==0
       direction = ( -Hessian + b_reference * epsilon * eye(size(Hessian,1)) ) \ score;
       direction = direction / norm(direction);
       step_para = direction * (direction'*score) / (direction' * (-Hessian) * direction);
              
       %translate step_para into original parameters:
       [step_beta,step_alpha,step_gamma,step_lambda,step_f] = para_transform1(step_para,K,N,T,R,Rex1,Rex2);
       
       %try to update parameters, but making sure that penalized objective
       %increases:
       fct=1;     %if we cannot make progress, then we try to make "a smaller step",
       %but normal step corresponds to fct=1
       stop=0;
       while stop==0
           para   = para0 + fct * step_para;
           beta   = beta0 + fct * step_beta;
           lambda = lambda0 + fct * step_lambda;
           alpha  = alpha0 + fct * step_alpha;
           f      = f0     + fct * step_f;
           gamma  = gamma0 + fct * step_gamma;
           
           obj     = SampleLogL(dist,Y,X,weight,lambda_known,f_known,beta,alpha,gamma,lambda,f);
           objPnew = obj - b_reference/2 * ((para-para_reference)' * v_reference) * (v_reference' * (para-para_reference));
           if objPnew>objP
               stop  = 1;
               stop2 = 1;
           else
               if (fct<1/2) && (epsilon < epsilon_max * 0.9999)
                 fct     = 1;
                 epsilon = max( epsilon_min, epsilon*10 ); 
                 stop    = 1;
               else
                  fct = fct/2;          %if we cannot make progress, then we reduce the "steplength"
                  if fct < 10^(-4)      %need some final stopping criterion
                     stop  = 1;
                     stop2 = 1;
                  end         
               end    
           end
       end
   end
   
   if objPnew < objP    %if no progress was made in the likelihood maximization 
     beta   = beta0;     %then just report the old parameters and objective
     alpha  = alpha0;
     gamma  = gamma0;
     f      = f0;
     lambda = lambda0;
     obj    = obj0;  
   end    
   
    fct
    epsilon
    
%    format long 
%    obj
     
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% perform one iteration step (similar to EM)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [beta,alpha,gamma,lambda,f,obj] = step(dist,Y,X,weight,lambda_known,f_known,beta0,alpha0,gamma0,lambda0,f0)
   
   K=size(X,1);
   N=size(X,2);
   T=size(X,3);   
   R=size(lambda0,2);
   
   fct=1;     %if we cannot make progress, then we try to make "a smaller step",
              %but normal step corresponds to fct=1
   stop=0;
   while stop==0
       %definition of single index:
       Z = mult(X,beta0) + alpha0 * f_known' + lambda_known * gamma0' + lambda0 * f0';
       
       %get corresponding log-likelihood and derivatives for each observation:
       [logL,dlogL,ddlogL] = model(dist,Y,Z,weight);
       obj0                = mean(mean(logL));
       
       %add dlogL./wt to single index
       Z = Z + fct * dlogL/mean(mean(-ddlogL));
              
       %PRINCIPAL COMPONENTS STEP TO GET NEW alpha,gamma,lambda,f:
       M_lambda_known = eye(N)-lambda_known/(lambda_known'*lambda_known)*lambda_known';
       M_f_known      = eye(T)-f_known/(f_known'*f_known)*f_known';
       res            = M_lambda_known * (Z - mult(X,beta0)) * M_f_known;
       
       [f,D]=eigs(res'*res,R);
       for r=1:R
           f(:,r)=f(:,r)/norm(f(:,r));
           if mean(f(:,r))<0
               f(:,r)=-f(:,r);
           end
       end
       lambda = res*f;
       f      = f*sqrt(T);
       lambda = lambda/sqrt(T);
       
       res    = Z - mult(X,beta0) - lambda*f';
       gamma  = res'*lambda_known/(lambda_known'*lambda_known);
       res    = res-lambda_known * gamma';
       alpha  = res*f_known/(f_known'*f_known);
       
       %update Z:
       Z                   = mult(X,beta0) + alpha * f_known' + lambda_known * gamma' + lambda * f';
       [logL,dlogL,ddlogL] = model(dist,Y,Z,weight);
       obj1                = mean(mean(logL));

       %WLS STEP TO GET NEW beta, POSSIBILITY 1:
       Z                   = Z + fct * dlogL./mean(mean(-ddlogL)); 
       lambda_all          = [lambda,lambda_known];
       f_all               = [f,f_known];
       Mlambda_all         = eye(N)-lambda_all/(lambda_all'*lambda_all)*lambda_all';
       Mf_all              = eye(T)-f_all/(f_all'*f_all)*f_all';
       W                   = zeros(K,K);
       V                   = zeros(K,1);
       for k1 = 1:K
           Xk1   = Mlambda_all*squeeze(X(k1,:,:))*Mf_all;
           V(k1) = 1/N/T*trace(Xk1*Z');
           for k2 = 1:K
               Xk2      = Mlambda_all*squeeze(X(k2,:,:))*Mf_all;
               W(k1,k2) = 1/N/T*trace(Xk1*Xk2');
           end
       end
       beta=W\V;
       
       %WLS STEP TO GET NEW beta, POSSIBILITY 2:
       
%        Z = mult(X,beta0) + fct * dlogL./max(-ddlogL,10^-7);
%        W=zeros(K,K);
%        V=zeros(K,1);
%        for k1=1:K
%            Xk1=squeeze(X(k1,:,:)) .* (-ddlogL);
%            V(k1)=1/N/T*trace(Xk1*Z');
%            for k2=1:K
%                Xk2=squeeze(X(k2,:,:));
%                W(k1,k2)=1/N/T*trace(Xk1*Xk2');
%            end
%        end
%        beta=W\V;
       
       obj2 = mean(mean(model(dist,Y,mult(X,beta) + alpha * f_known' + lambda_known * gamma' + lambda * f',weight)));
       if obj2 > obj0
         stop=1;               %We stop if the we indeed increased the likelihood function in this step.
       else                    %If we have not increased the likelihood:
           fct=fct/2;          %then we reduce the "steplength"
           if fct<10^(-4)      %need some final stopping criterion
             stop=1;          
           end
       end  
   end
   
   if obj2 > obj0    %if progress was made in the likelihood maximization         
     obj = obj2;       %then report the new objective and updated parameter
   else              %otherwise 
     beta  = beta0;     %report the old parameters and objective
     alpha = alpha0;
     gamma = gamma0;
     f     = f0;
     lambda= lambda0;
     obj   = obj0;  
   end    
   
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% multiplication between beta and X to form NxT matrix:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mat = mult(X,beta)
  mat=zeros(size(X,2),size(X,3));
  for k=1:size(X,1)
    mat=mat+beta(k)*squeeze(X(k,:,:));
  end
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  cdf of normal distribution, and its derivative:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function p=normal_cdf(x)
  p=normcdf(x,0,1);
return  

function p=normal_pdf(x)
  p=normpdf(x,0,1);
return  

function p=normal_pdf_derivative(x)   
  p=-x.*normpdf(x,0,1);
return  


